<?php

namespace Drupal\cryptophoto\Form;

//use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\User;

/**
 * CryptoPhoto Setting page.
 */
class CryptoPhotoSettingsForm extends ConfigFormBase {
  public function getFormId() {
    return 'cryptophoto_settings_form';
  }

  protected function getEditableConfigNames() {
    return ['cryptophoto.settings'];
  } 

  public function buildForm(array $form, FormStateInterface $form_state) {
    // get config
    $config = $this->config('cryptophoto.settings');

    // wizard button for autoconfiguration
    $form['wizard'] = array(
      '#type' => 'button',
      '#attributes' => array('onclick' => 'CPPluginWizard.start_wizard(); return false'),
      '#value' => t("Wizard"),
    );

    // public key
    $form['publickey'] = array(
      '#id' => 'publickey',
      '#type' => 'textfield',
      '#title' => $this->t('Public Key'),
      '#maxlength' => 255,
      '#default_value' => $config->get('publickey'),
      '#description' => $this->t("Your CryptoPhoto API Public Key"),
      '#required' => TRUE,
    );

    // private key
    $form['privatekey'] = array(
      '#id' => 'privatekey',
      '#type' => 'textfield',
      '#title' => $this->t('Private Key'),
      '#maxlength' => 255,
      '#default_value' => $config->get('privatekey'),
      '#description' => $this->t("Your CryptoPhoto API Private Key"),
      '#required' => TRUE,
    );

    // salt
    $form['salt'] = array(
      '#id' => 'salt',
      '#type' => 'textfield',
      '#title' => $this->t('Salt'),
      '#maxlength' => 255,
      '#default_value' => $config->get('salt'),
      '#description' => $this->t("The 'Salt' is used to create unique user IDs. It is recomended to use a random string and once set, not to change it, otherwise the CryptoPhoto settings of each of your users will be reset"),
      '#required' => TRUE,
    );

    // server
    $server = $config->get('server');
    if (!$server) $server = 'https://cryptophoto.com';
    $form['server'] = array(
      '#id' => 'server',
      '#type' => 'textfield',
      '#title' => $this->t('Server'),
      '#maxlength' => 255,
      '#default_value' => $server,
      '#description' => $this->t("Set a server url if you're using a CryptoPhoto appliance, otherwise leave this field as is"),
      '#required' => TRUE,
    );

    $roles = user_role_names();
    unset($roles['anonymous']);

    $allowed = $config->get('allowed_roles');
    if ($allowed == null) $allowed = array_keys($roles);

    // roles allowed to use CryptoPhoto
    $form['allowed_roles'] = array(
      '#type'          => 'checkboxes',
      '#title'         => $this->t('CryptoPhoto allowed roles'),
      '#default_value' => $allowed,
      '#options'       => $roles,
      '#description'   => $this->t('Users with the specified roles will be allowed to use Cryptophoto'),
    );

    // roles required to use Cryptophoto
    $form['required_roles'] = array(
      '#type'          => 'checkboxes',
      '#title'         => $this->t('CryptoPhoto required roles'),
      '#default_value' => $config->get('required_roles'),
      '#options'       => $roles,
      '#description'   => $this->t('Users with the specified roles will be required to use Cryptophoto'),
    );

    // AJAX CryptoPhoto configuration test
    $form['testresult'] = array(
      '#id' => 'testresult',
      '#type' => 'container',
    );

    // form actions
    $form['actions']['#type'] = 'actions';
    $form['actions']['save'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    );
    $form['actions']['test_cfg'] = array (
      '#type' => 'button',
      '#attributes' => array('onclick' => 'testConfig(); return false'),
      '#value' => t("Test Configuration"),
    );

    // get server IP, this is used for the CryptoPhoto autoconfiguration
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_HEADER, false);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_URL, 'https://cryptophoto.com/show_my_ip');
    curl_setopt($ch, CURLOPT_REFERER, 'https://cryptophoto.com/show_my_ip');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $svip = curl_exec($ch);
    curl_close($ch);

    //$svip = file_get_contents('https://cryptophoto.com/show_my_ip');

    // attach JavaScript
    $form['#attached']['library'][] = 'cryptophoto/cp_test_script';
    $form['#attached']['drupalSettings']['cryptophoto']['svip'] = $svip;
    global $base_url;
    $form['#attached']['drupalSettings']['cryptophoto']['base_url'] = $base_url;

    return $form;
  }

  /*
   * User input validation
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // get values
    $values = $form_state->getValues();
    
    $publickey = $values['publickey'];
    $privatekey = $values['privatekey'];
    $salt = $values['salt'];
    $server = $values['server'];

    // test
    if (strlen($publickey) == 0) {
      $form_state->setErrorByName('publickey', $this->t("You must provide the CryptoPhoto public key"));
    }

    if (strlen($privatekey) == 0) {
      $form_state->setErrorByName('privatekey', $this->t("You must provide the CryptoPhoto private key"));
    }

    if (strlen($salt) == 0) {
      $form_state->setErrorByName('salt', $this->t("You must the CryptoPhoto salt"));
    }

    if (strlen($server) == 0) {
      $form_state->setErrorByName('server', $this->t("You must provide the CryptoPhoto server"));
    }

    parent::validateForm($form, $form_state);
  }

  /*
   * Save user input
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    // get values
    $values = $form_state->getValues();
    
    $publickey = $values['publickey'];
    $privatekey = $values['privatekey'];
    $salt = $values['salt'];
    $server = $values['server'];

    // get CryptoPhoto configuration
    $config = $this->config('cryptophoto.settings');

    // set new values
    $config->set('publickey', $publickey);
    $config->set('privatekey', $privatekey);
    $config->set('salt', $salt);
    $config->set('server', $server);

    if ($values['allowed_roles']) {
      $rv = array();
      $av = array_values($values['allowed_roles']);
      foreach ($av as $v) { if ($v) array_push($rv, $v); }
      $config->set('allowed_roles', $rv);
    }

    if ($values['required_roles']) {
      $rv = array();
      $av = array_values($values['required_roles']);
      foreach ($av as $v) { if ($v) array_push($rv, $v); }
      $config->set('required_roles', $rv);
    }

    // save
    $config->save();
  }

}
